/*
 * NRF24L01.h
 *
 * Created: 5. 6. 2015 12:05:02
 *  Author: Ondra
 */ 

//Knihovna ovldn 2,4GHz modulu NRF

#ifndef NRF24L01_H_
#define NRF24L01_H_

//#ifdef _HAVE_CONFIG
#include "config.h"
//#endif

// Definice ---------------------------------------------------------------------
#ifndef NRF_SELECT
#error NRF_SELECT not defined!
#endif

#ifndef NRF_UNSELECT
#error NRF_UNSELECT not defined!
#endif

#ifndef NRF_ENABLED
#error NRF_ENABLED not defined!
#endif

#ifndef NRF_DISABLED
#error NRF_DISABLED not defined!
#endif

#ifndef NRF_IRQ
#error NRF_IRQ not defined!
#endif

#ifndef NRF_SPI
#error NRF_SPI must be defined!
#endif

#ifndef NRF_RX_TIMEOUT
#define NRF_RX_TIMEOUT		5	// [ms]
#endif

#ifndef NRF_RETRANSMITS
#define NRF_RETRANSMITS		3
#endif

#ifndef NRF_MAX_PACKET_LEN		// Split packets in buffer into smaller pieces
#define NRF_MAX_PACKET_LEN	32	// [bytes]
#endif

// Enums ------------------------------------------------------------------------
typedef enum
{
	NRF_PRX,		// Primary RX mode
	NRF_PTX,		// Primary TX mode
	NRF_SLEEP,		// Power down
} e_nrf_mode;

// Structs ----------------------------------------------------------------------
#ifndef NRF_STATUS_DISABLED
typedef struct
{
	unsigned long tx_cnt;		// Packets
	unsigned long tx_fail_cnt;	// Packets
	unsigned long rx_cnt;		// Packets
} T_NRF_status;

typedef struct
{
	unsigned short addr_src;
	unsigned short addr_dest;
	unsigned char rq_st;
	unsigned char data_len;
	unsigned char * buffer;
} T_NRF_Packet;

extern T_NRF_status nrf_pck_status;
#endif	// NRF_STATUS_DISABLED

// Commands ---------------------------------------------------------------------
#define R_REGISTER			0b00000000
#define W_REGISTER			0b00100000
#define W_TX_PAYLOAD		0b10100000
#define R_RX_PAYLOAD		0b01100001
#define FLUSH_TX			0b11100001
#define FLUSH_RX			0b11100010
#define REFUSE_TX_PL		0b11100011
#define ACTIVATE			0b01010000
#define R_RX_PL_WID			0b01100000
#define W_ACK_PAYLOAD		0b10101000
#define	W_TX_PAYLOAD_NO_ACK	0b10110000
#define NRF_NOP				0b11111111

// NRF registers ----------------------------------------------------------------
#define REG_ADDR_MASK		0b00011111
#define NRF_CONFIG			0x00
#define NRF_EN_AA			0x01
#define NRF_EN_RXADDR		0x02
#define NRF_SETUP_AW		0x03
#define NRF_SETUP_RETR		0x04
#define	NRF_RF_CH			0x05
#define NRF_RF_SETUP		0x06
#define NRF_STATUS			0x07
#define NRF_OBSERVE_TX		0x08
#define NRF_CD				0x09
#define NRF_RX_ADDR_P0		0x0A
#define NRF_RX_ADDR_P1		0x0B
#define NRF_RX_ADDR_P2		0x0C
#define NRF_RX_ADDR_P3		0x0D
#define NRF_RX_ADDR_P4		0x0E
#define NRF_RX_ADDR_P5		0x0F
#define NRF_TX_ADDR			0x10
#define NRF_RX_PW_P0		0x11	// Number of bytes in RX payload
#define NRF_RX_PW_P1		0x12
#define NRF_RX_PW_P2		0x13
#define NRF_RX_PW_P3		0x14
#define NRF_RX_PW_P4		0x15
#define NRF_RX_PW_P5		0x16
#define NRF_FIFO_STATUS		0x17	// FIFO status register
#define NRF_DYNPD			0x1C	// Enable dynamic payload length to each pipe
#define NRF_FEATURE			0x1D	

// Status register bits mask
#define RX_DR				(1 << 6)
#define TX_DS				(1 << 5)
#define MAX_RT				(1 << 4)
#define RX_P_NO				(7 << 1)
#define TX_FULL				(1 << 0)

extern uint8_t nrf_r_register(uint8_t address, uint8_t *ret_data, uint8_t data_len);
extern uint8_t nrf_w_register(uint8_t address, uint8_t *data, uint8_t data_len);
extern uint8_t nrf_w_register_byte(uint8_t address, uint8_t data);
extern void nrf_w_payload(unsigned char * ptr, unsigned char data_len, bool ack_dis);
extern void nrf_r_payload(unsigned char * ptr, unsigned char lenght);
extern unsigned char nrf_get_payload_width(void);
extern void nrf_flush_rx(void);
extern void nrf_flush_tx(void);
extern void nrf_init(unsigned long int device_address, unsigned char freq_channel);
extern void nrf_change_mode(e_nrf_mode nrf_mode);		// 1 - receiver, 0 - transmitter
extern unsigned char nrf_get_status(void);
extern void nrf_ms_tick(void);
extern char nrf_send_pck(T_NRF_Packet *pck);
extern char nrf_send_pck_no_ack(T_NRF_Packet *pck);
extern char nrf_pck_detect(T_NRF_Packet *pck,  unsigned short my_addr);


#endif /* NRF24L01_H_ */
