#include "adc_meas.h"
#include "config.h"

#include <adc.h>
#include <stdint.h>
#include <stdio.h>
#include <fsl_pmc_hal.h>
#include <fsl_adc16_hal.h>

// --------------------------------------------------------------------------
/*global variable */
ADC_work_struct adc;

// --------------------------------------------------------------------------
void adc_init(void)
{
	pmc_bandgap_buffer_config_t bandgap_cfg;

	bandgap_cfg.enable = true;
	bandgap_cfg.enableInLowPower = false;

	PMC_HAL_BandgapBufferConfig(PMC, &bandgap_cfg);
	init_adc(kAdc16ClkDividerOf4, false);
	ADC16_HAL_SetChnMuxMode(ADC0, kAdc16ChnMuxOfB);
	set_adc_averaging(true, kAdc16HwAverageCountOf16);
	adc.adc_running = false;
}

// --------------------------------------------------------------------------
void adc_run(void)
{
	if (adc.adc_running)
		return;
	run_adc(ADC_CH_BATTERY, 0);		// Start measuring internal bandgap voltage
	adc.adc_running = true;
}

// --------------------------------------------------------------------------
void adc_handle(void)
{
	if (get_adc_done() && adc.adc_running)
	{
		adc.adc_running = false;
		st.voltage = (1000 * 65536) / get_adc(ADC_BATTERY);
	}
}

// --------------------------------------------------------------------------
bool adc_get_running(void)
{
	return adc.adc_running;
}

// --------------------------------------------------------------------------
bool adc_check_battery(void)
{
	LedR_Set();

	switch_clk(E_CLK_FAST_MODE);		// To increase power consumption
	adc_run();

	while (adc_get_running())
	{
		adc_handle();
	}
	LedR_Clear();
	switch_clk(E_CLK_SLOW_MODE);

	if (st.voltage < BATT_MIN_VOLTAGE)	// Battery voltage is very low
		return false;

	return true;
}

