#define COLORED     0
#define UNCOLORED   1

#include "ui.h"
#include "config.h"
#include "main.h"
#include "adc_meas.h"

#include <e_ink_display.h>
#include <epdpaint.h>
#include <fonts.h>
#include <stdio.h>
#include <string.h>

// Global variables -----------------------------
UI_struct ui;
UI_Message_struct message;
uint8_t image[2048];

// ----------------------------------------------

// Active connection symbol, 9x16px
const uint8_t symb_9px[] =
{
	// Connection active
	0b10010010, 0b00000100,
	0b01010100, 0b00000010,
	0b00111000, 0b11111111,
	0b00010000, 0b00000010,
	0b00010000, 0b00100100,
	0b00010000, 0b01000000,
	0b00010000, 0b11111110,
	0b00010000, 0b01000000,
	0b00010000, 0b00100000,

	// Connection inactive
	0b10010010, 0b00000000,
	0b01010100, 0b00000000,
	0b00111000, 0b00100010,
	0b00010000, 0b00010100,
	0b00010000, 0b00001000,
	0b00010000, 0b00010100,
	0b00010000, 0b00100010,
	0b00010000, 0b00000000,
	0b00010000, 0b00000000
};

sFONT FontSymb9 = {
	symb_9px,
	16, /* Width */
	9, /* Height */
};

// Render test screen
void ui_test_screen(void)
{
	// No test screen
}

// Set page extended
void set_page_ex(e_ui_page page, uint8_t selected_item)
{
	ui.page = page;
	ui.screen_update = true;
	ui.partial_update = false;
	ui.selected_item = selected_item;
}

// Set page
void set_page(e_ui_page page)
{
	set_page_ex(page, 0);
}

// Update page
void update_page(void)
{
	ui.screen_update = true;
	ui.partial_update = true;
}

// Clear button disable scan timer
void ui_button_tmr_clear(void)
{
	ui.btn_time = 0;
}

// Scanning pushed buttons
void ui_button_scan(void)
{
	if (ui.btn_time)
		return;

	// Scan buttons
	if (SW_DOWN_Read() == 0)
	{
		ui.btn_pressed = BTN_DOWN;
		ui.btn_time = BUTTON_TIME;
	}
	else if (SW_UP_Read() == 0)
	{
		ui.btn_pressed = BTN_UP;
		ui.btn_time = BUTTON_TIME;
	}
	else if (SW_OK_Read() == 0)
	{
		ui.btn_pressed = BTN_OK;
		ui.btn_time = BUTTON_TIME;
	}
	// -------------------
}

// Tick for decrementing timers
void ui_ms_tick(void)
{
	// Button scan delay timer
	if (ui.btn_time)
		ui.btn_time--;
}

// Handle timers. This function should be called each second
// Return: true if some of timer is running
void ui_sec_tick(void)
{
	// Show buttons on screen timer
	if (ui.btn_on_screen)
	{
		second_wake_enable();
		if (--ui.btn_on_screen == 0)
		{
			ui.screen_update = true;
			ui.partial_update = true;
		}
	}

	// Automatically return to main screen
	if (ui.return_timer)
	{
		second_wake_enable();
		if (--ui.return_timer == 0)
		{
			set_page(GUI_MAIN);
		}
	}

	if (message.timer)
	{
		second_wake_enable();
		if (--message.timer == 0)
		{
			set_page_ex(message.page, message.selected_item);
		}
	}
}

// Show butons on screen
void show_buttons(void)
{
	ui.btn_on_screen = BUTTON_ONSCREEN_TIME;
	ui.screen_update = true;
	ui.partial_update = true;
}

// Show message on screen
void show_message(char *text, uint8_t time, e_ui_page next_page, uint8_t next_selected_item)
{
	strncpy(message.text, text, sizeof(message.text) - 1);
	message.text[sizeof(message.text) - 1] = 0;
	message.page = next_page;
	message.selected_item = next_selected_item;
	message.timer = time;
	update_page();
}

// Handle user pressed buttons
void buttons_handle(void)
{
	if (ui.btn_pressed == BTN_NONE)
		return;

	// Buttons works only if labels is shown
	if (ui.btn_on_screen)
	{
		ui.btn_on_screen = BUTTON_ONSCREEN_TIME;
		switch (ui.page)
		{
			// Main page with actual measured values
			case GUI_MAIN :
				switch (ui.btn_pressed)
				{
					case BTN_OK:		// Menu
						set_page(GUI_MENU);
						break;

					case BTN_UP:		// Max values
						set_page(GUI_MAIN_MAX);
						break;

					case BTN_DOWN:		// Min values
						set_page(GUI_MAIN_MIN);
						break;

					default:
						break;
				}
				break;

			// Page with maximum and minimum values
			case GUI_MAIN_MIN:
			case GUI_MAIN_MAX:
				switch (ui.btn_pressed)
				{
					case BTN_OK:		// Main page
						set_page(GUI_MAIN);
						break;

					case BTN_UP:		// Max values
						if (ui.page == GUI_MAIN_MIN)
							set_page(GUI_MAIN_MAX);
						break;

					case BTN_DOWN:		// Min values
						if (ui.page == GUI_MAIN_MAX)
							set_page(GUI_MAIN_MIN);
						break;

					default:
						break;
				}
				break;

			// Page with Menu screen
			case GUI_MENU:
				switch (ui.btn_pressed)
				{
					// Handle selected item in menu
					case BTN_OK:
						switch (ui.selected_item)
						{
							case 0:		// Graph page
								set_page(GUI_GRAPH_TEMP);
								break;

							case 1:		// Clear statistics
								set_page_ex(GUI_CLR_STATS, 0);
								break;

							case 3:		// Main page
								set_page(GUI_MAIN);
								break;

							default:
								break;
						}
						break;

					// Move cursor UP
					case BTN_UP:		// UP
						if (ui.selected_item > 0)
						{
							ui.selected_item--;
							update_page();
						}
						break;

					// Move cursor DOWN
					case BTN_DOWN:		// DOWN
						if (ui.selected_item < 3)
						{
							ui.selected_item++;
							update_page();
						}
						break;

					default:
						break;
				}
				break;

			// Page for clear statistics
			case GUI_CLR_STATS:
				switch (ui.btn_pressed)
				{
					case BTN_OK:
						if (ui.selected_item == 0) 	// Do not clear statistics
						{
							set_page_ex(GUI_MENU, 1);
						}
						else						// Clear statistics
						{
							show_message("CLEARED", 3, GUI_MENU, 1);
							measure_init();
						}
						break;

					case BTN_UP:		// UP
						if (ui.selected_item > 0)
						{
							ui.selected_item--;
							update_page();
						}
						break;

					case BTN_DOWN:		// DOWN
						if (ui.selected_item < 1)
						{
							ui.selected_item++;
							update_page();
						}
						break;

					default:
						break;
				}
				break;

			case GUI_GRAPH_TEMP:
				switch (ui.btn_pressed)
				{
					case BTN_OK:
						set_page_ex(GUI_MENU, 0);
						break;

					case BTN_UP:		// UP
						set_page_ex(GUI_GRAPH_HUMID, 0);
						break;

					case BTN_DOWN:		// DOWN
						set_page_ex(GUI_GRAPH_PRESS, 0);
						break;

					default:
						break;
				}
				break;

			case GUI_GRAPH_HUMID:
				switch (ui.btn_pressed)
				{
					case BTN_OK:
						set_page_ex(GUI_MENU, 0);
						break;

					case BTN_UP:		// UP
						set_page_ex(GUI_GRAPH_TEMP, 0);
						break;

					case BTN_DOWN:		// DOWN
						set_page_ex(GUI_GRAPH_PRESS, 0);
						break;

					default:
						break;
				}
				break;

			case GUI_GRAPH_PRESS:
				switch (ui.btn_pressed)
				{
					case BTN_OK:
						set_page_ex(GUI_MENU, 0);
						break;

					case BTN_UP:		// UP
						set_page_ex(GUI_GRAPH_HUMID, 0);
						break;

					case BTN_DOWN:		// DOWN
						set_page_ex(GUI_GRAPH_TEMP, 0);
						break;

					default:
						break;
				}
				break;

			default:
				break;
		}

		// If screen is not in Main page, set timer for automatic return
		if (ui.page != GUI_MAIN)
			ui.return_timer = UI_RETURN_TIME;
		else
			ui.return_timer = 0;
	}
	else
		show_buttons();

	ui.btn_pressed = BTN_NONE;
}

// Render header with time, signal and battery level
void render_header(void)
{
	RTC_struct rtc;
	uint16_t battery;
	char text[20];

	sec_to_time(&rtc, st.rtc_time);

	paint_SetRotate(ROTATE_0);
	paint_SetWidth(64);
	paint_SetHeight(16);

	sprintf(text, "%i:%02i", rtc.hours, rtc.minutes);

	paint_Clear(COLORED);
	paint_DrawStringAt(0, 3, text, &Font12, UNCOLORED);
	Epd_SetFrameMemoryPart(paint_GetImage(), 0, 0, paint_GetWidth(), paint_GetHeight());

	if (st.voltage > BATT_MIN_VOLTAGE)
		battery = (st.voltage - BATT_MIN_VOLTAGE) * 100 / (BATT_MAX_VOLTAGE - BATT_MIN_VOLTAGE);
	else
		battery = 0;

	if (battery > 100)
		battery = 100;

	sprintf(text, "%i%%", battery);

	paint_Clear(COLORED);
	paint_DrawStringAt(35, 3, text, &Font12, UNCOLORED);
	Epd_SetFrameMemoryPart(paint_GetImage(), 64, 0, paint_GetWidth(), paint_GetHeight());

	text[0] = ' ' + !(st.server_connected);
	text[1] = 0;

	paint_SetWidth(20);
	paint_SetHeight(16);
	paint_Clear(COLORED);
	paint_DrawStringAt(0, 3, text, &FontSymb9, UNCOLORED);
	Epd_SetFrameMemoryPart(paint_GetImage(), 60, 0, paint_GetWidth(), paint_GetHeight());
}

// Render button labels
void render_buttons(const char *up_text, const char *ok_text, const char *down_text)
{
	uint8_t length;

	paint_SetRotate(ROTATE_270);
	paint_SetWidth(16);
	paint_SetHeight(48);

	// Up
	if (up_text != NULL && *up_text != 0)
	{
		length = strlen(up_text);
		paint_Clear(UNCOLORED);
		paint_DrawRectangle(1, 0, 47, 15, COLORED);
		paint_DrawStringAt((48 / 2) - ((length * Font12.Width) / 2), 3, up_text, &Font12, COLORED);
		Epd_SetFrameMemoryPart(paint_GetImage(), 112, 56, paint_GetWidth(), paint_GetHeight());
	}

	// Ok
	if (ok_text != NULL && *ok_text != 0)
	{
		length = strlen(ok_text);
		paint_Clear(UNCOLORED);
		paint_DrawRectangle(1, 0, 47, 15, COLORED);
		paint_DrawStringAt((48 / 2) - ((length * Font12.Width) / 2), 3, ok_text, &Font12, COLORED);
		Epd_SetFrameMemoryPart(paint_GetImage(), 112, 128, paint_GetWidth(), paint_GetHeight());
	}

	// Down
	if (down_text != NULL && *down_text != 0)
	{
		length = strlen(down_text);
		paint_Clear(UNCOLORED);
		paint_DrawRectangle(1, 0, 47, 15, COLORED);
		paint_DrawStringAt((48 / 2) - ((length * Font12.Width) / 2), 3, down_text, &Font12, COLORED);
		Epd_SetFrameMemoryPart(paint_GetImage(), 112, 200, paint_GetWidth(), paint_GetHeight());
	}
}

// Render menu title
void render_menu_title(const char *text)
{
	paint_SetRotate(ROTATE_0);
	paint_SetWidth(128);
	paint_SetHeight(24);

	paint_Clear(COLORED);
	paint_DrawStringAt(3, 3, text, &Font24, UNCOLORED);
	Epd_SetFrameMemoryPart(paint_GetImage(), 0, 24, paint_GetWidth(), paint_GetHeight());
}

// Render menu item
void render_menu_item(const char *text, uint8_t position, uint8_t selected)
{
	paint_SetWidth(128);
	paint_SetHeight(16);
	paint_Clear(!selected);
	paint_DrawStringAt(3, 3, text, &Font12, !!selected);
	Epd_SetFrameMemoryPart(paint_GetImage(), 0, 48 + (position * 16), paint_GetWidth(), paint_GetHeight());
}

// Render menu page
void render_menu_page(void)
{
	Epd_ClearFrameMemory(0xFF);
	render_header();

	render_menu_title("MENU");

	render_menu_item("Graphs", 0, ui.selected_item == 0);
	render_menu_item("Clear max/min", 1, ui.selected_item == 1);
	render_menu_item("Configuration", 2, ui.selected_item == 2);
	render_menu_item("Return", 3, ui.selected_item == 3);

	if (ui.btn_on_screen)
		render_buttons("->", "OK", "<-");
}

// Render clear statistics page
void render_clr_stat_page(void)
{
	Epd_ClearFrameMemory(0xFF);
	render_header();

	render_menu_title("SURE?");

	paint_SetWidth(128);
	paint_SetHeight(48);
	paint_Clear(UNCOLORED);
	paint_DrawStringAt(3, 3, "Do you really", &Font12, COLORED);
	paint_DrawStringAt(3, 19, "want to clear", &Font12, COLORED);
	paint_DrawStringAt(3, 35, "max/min stats?", &Font12, COLORED);
	Epd_SetFrameMemoryPart(paint_GetImage(), 0, 48, paint_GetWidth(), paint_GetHeight());

	render_menu_item("No, I do not", 4, ui.selected_item == 0);
	render_menu_item("Yes, I do", 5, ui.selected_item == 1);

	if (ui.btn_on_screen)
		render_buttons("->", "OK", "<-");
}

// Render graphs
void render_graph_page(e_ui_page page)
{
	TMeasureRec min_vals, max_vals;
	int32_t min_graph, max_graph;
	int32_t difference;
	char text[20];

	Epd_ClearFrameMemory(0xFF);
	render_header();
	render_menu_title("GRAPHS");

	// Draw graphs titles
	paint_SetWidth(128);
	paint_SetHeight(16);
	paint_Clear(UNCOLORED);
	if (page == GUI_GRAPH_TEMP) paint_DrawStringAt(0, 3, "Temp. IN/24h", &Font12, COLORED);
	if (page == GUI_GRAPH_HUMID) paint_DrawStringAt(0, 3, "Humid. IN/24h", &Font12, COLORED);
	if (page == GUI_GRAPH_PRESS) paint_DrawStringAt(0, 3, "Press. IN/24h", &Font12, COLORED);
	Epd_SetFrameMemoryPart(paint_GetImage(), 0, 48, paint_GetWidth(), paint_GetHeight());

	paint_SetWidth(128);
	paint_SetHeight(16);
	paint_Clear(UNCOLORED);
	if (page == GUI_GRAPH_TEMP) paint_DrawStringAt(0, 3, "Temp. OUT/24h", &Font12, COLORED);
	if (page == GUI_GRAPH_HUMID) paint_DrawStringAt(0, 3, "Humid. OUT/24h", &Font12, COLORED);
	if (page == GUI_GRAPH_PRESS) paint_DrawStringAt(0, 3, "Press. OUT/24h", &Font12, COLORED);
	Epd_SetFrameMemoryPart(paint_GetImage(), 0, 176, paint_GetWidth(), paint_GetHeight());

	// Find maximum and minimum values in 24h history
	measure_find_max_min(&max_vals, &min_vals);

	// Select graph scale
	difference = max_vals.temperature_in - min_vals.temperature_in;
	if (difference > 1000)
	{	// For difference more than 10 C
		min_graph = (min_vals.temperature_in / 1000) * 1000;
		max_graph = min_graph + ((difference / 1000) + 1) * 1000;
	}
	else
	{	// For difference less than 10 C
		min_graph = (min_vals.temperature_in / 500) * 500;
		max_graph = min_graph + ((difference / 500) + 1) * 500;
	}

	// Draw axis ranges
	paint_SetWidth(32);
	paint_SetHeight(96);
	paint_Clear(UNCOLORED);

	sprintf(text, "%ld,%02ld", max_graph / 100, max_graph % 100);
	paint_DrawStringAt(0, 2, text, &Font8, COLORED);
	sprintf(text, "%ld,%02ld", (max_graph + min_graph) / 200, ((max_graph + min_graph) / 2) % 100);
	paint_DrawStringAt(0, 42, text, &Font8, COLORED);
	sprintf(text, "%ld,%02ld", min_graph / 100, min_graph % 100);
	paint_DrawStringAt(0, 82, text, &Font8, COLORED);

	for (uint8_t i = 0; i < 11; i++)
		paint_DrawHorizontalLine(29, (i * 8) + 4, 3, COLORED);

	Epd_SetFrameMemoryPart(paint_GetImage(), 0, 64, paint_GetWidth(), paint_GetHeight());

	// Draw graph axis
	paint_SetHeight(96);
	paint_SetWidth(96);
	paint_Clear(UNCOLORED);

	paint_DrawHorizontalLine(0, 90, 96, COLORED);
	paint_DrawVerticalLine(0, 0, 90, COLORED);

	// Draw Graph
	uint8_t y_pos;		// Variables to store computed y position on graph
	for (uint8_t i = 0; i < MEASURE_HIST_DEPTH; i++)
	{
		if (meas_history[(MEASURE_HIST_DEPTH - 1) - i].temperature_in != INT16_MAX)
		{
			y_pos = 84 - (80 * (meas_history[(MEASURE_HIST_DEPTH - 1) - i].temperature_in - min_graph) / (max_graph - min_graph));
			paint_DrawPixel(i, y_pos, COLORED);
		}
	}
	Epd_SetFrameMemoryPart(paint_GetImage(), 32, 64, paint_GetWidth(), paint_GetHeight());

	// Draw second graph
	// TODO:
	paint_Clear(UNCOLORED);

	// Draw graph axes
	paint_DrawHorizontalLine(0, 95, 96, COLORED);
	paint_DrawVerticalLine(0, 0, 96, COLORED);
	Epd_SetFrameMemoryPart(paint_GetImage(), 32, 192, paint_GetWidth(), paint_GetHeight());

	// Select scale for graph
	if (ui.btn_on_screen)
	{
		switch (page)
		{
			case GUI_GRAPH_TEMP:
				render_buttons("Humid.", "BACK", "Press.");
				break;
			case GUI_GRAPH_HUMID:
				render_buttons("Temp.", "BACK", "Press.");
				break;
			case GUI_GRAPH_PRESS:
				render_buttons("Humid.", "BACK", "Temp.");
				break;
			default:
				break;
		}
	}
}

void render_message(void)
{
	uint8_t length;

	paint_SetRotate(ROTATE_0);
	paint_SetWidth(128);
	paint_SetHeight(48);

	length = strlen(message.text);
	paint_Clear(UNCOLORED);
	paint_DrawRectangle(0, 0, 127, 47, COLORED);
	paint_DrawStringAt((128 / 2) - ((length * Font24.Width) / 2), 12, message.text, &Font24, COLORED);
	Epd_SetFrameMemoryPart(paint_GetImage(), 0, 128, paint_GetWidth(), paint_GetHeight());
}

void render_main_page(e_ui_page page)
{
	char text[25];
	int16_t temperature;
	uint16_t humidity;
	uint32_t pressure;

	Epd_ClearFrameMemory(0xFF);
	render_header();

	// Render temperature and humidity
	paint_SetRotate(ROTATE_0);
	paint_SetWidth(128);
	paint_SetHeight(24);

	// Indoor
	paint_Clear(COLORED);
	if (page == GUI_MAIN_MAX)
	{
		sprintf(text, "In(MAX)");
		temperature = st.temperature_in_max;
		humidity = st.humidity_in_max;
		pressure = st.pressure_in_max;
	}
	else if (page == GUI_MAIN_MIN)
	{
		sprintf(text, "In(MIN)");
		temperature = st.temperature_in_min;
		humidity = st.humidity_in_min;
		pressure = st.pressure_in_min;
	}
	else
	{
		sprintf(text, "Indoor");
		temperature = st.temperature_in;
		humidity = st.humidity_in;
		pressure = st.pressure_in;
	}
	paint_DrawStringAt(0, 3, text, &Font24, UNCOLORED);
	Epd_SetFrameMemoryPart(paint_GetImage(), 0, 24, paint_GetWidth(), paint_GetHeight());

	paint_SetHeight(16);
	paint_Clear(UNCOLORED);
	paint_DrawStringAt(0, 3, "Temperature:", &Font12, COLORED);
	Epd_SetFrameMemoryPart(paint_GetImage(), 0, 48, paint_GetWidth(), paint_GetHeight());

	paint_SetHeight(24);
	paint_Clear(UNCOLORED);
	sprintf(text, "%i,%iC", temperature / 100, (temperature / 10) % 10);
	paint_DrawStringAt(15, 3, text, &Font24, COLORED);
	Epd_SetFrameMemoryPart(paint_GetImage(), 0, 64, paint_GetWidth(), paint_GetHeight());

	paint_SetHeight(16);
	paint_Clear(UNCOLORED);
	paint_DrawStringAt(0, 3, "Humidity:", &Font12, COLORED);
	Epd_SetFrameMemoryPart(paint_GetImage(), 0, 88, paint_GetWidth(), paint_GetHeight());

	paint_SetHeight(24);
	paint_Clear(UNCOLORED);
	sprintf(text, "%i,%i", humidity / 100, (humidity / 10) % 10);
	paint_DrawStringAt(15, 3, text, &Font24, COLORED);
	Epd_SetFrameMemoryPart(paint_GetImage(), 0, 104, paint_GetWidth(), paint_GetHeight());

	paint_SetHeight(16);
	paint_Clear(UNCOLORED);
	paint_DrawStringAt(0, 3, "Pressure:", &Font12, COLORED);
	Epd_SetFrameMemoryPart(paint_GetImage(), 0, 128, paint_GetWidth(), paint_GetHeight());

	paint_SetHeight(16);
	paint_Clear(UNCOLORED);
	sprintf(text, "%i,%i hPa", (int)(pressure / 100), (int)(pressure % 100));
	paint_DrawStringAt(0, 3, text, &Font12, COLORED);
	Epd_SetFrameMemoryPart(paint_GetImage(), 24, 144, paint_GetWidth(), paint_GetHeight());

	// Outdoor
	paint_SetHeight(24);
	paint_Clear(COLORED);
	if (page == GUI_MAIN_MAX)
	{
		sprintf(text, "Out(MAX)");
		temperature = st.temperature_out_max;
		humidity = st.humidity_out_max;
		pressure = st.pressure_out_max;
	}
	else if (page == GUI_MAIN_MIN)
	{
		sprintf(text, "Out(MIN)");
		temperature = st.temperature_out_min;
		humidity = st.humidity_out_min;
		pressure = st.pressure_out_min;
	}
	else
	{
		sprintf(text, "Outdoor");
		temperature = st.temperature_out;
		humidity = st.humidity_out;
		pressure = st.pressure_out;
	}
	paint_DrawStringAt(0, 3, text, &Font24, UNCOLORED);
	Epd_SetFrameMemoryPart(paint_GetImage(), 0, 160, paint_GetWidth(), paint_GetHeight());

	paint_SetHeight(16);
	paint_Clear(UNCOLORED);
	paint_DrawStringAt(0, 3, "Temperature:", &Font12, COLORED);
	Epd_SetFrameMemoryPart(paint_GetImage(), 0, 184, paint_GetWidth(), paint_GetHeight());

	paint_SetHeight(24);
	paint_Clear(UNCOLORED);
	sprintf(text, " N/A");
	//sprintf(text, "%i,%iC", sensor.temp / 10, sensor.temp % 10);
	paint_DrawStringAt(15, 3, text, &Font24, COLORED);
	Epd_SetFrameMemoryPart(paint_GetImage(), 0, 200, paint_GetWidth(), paint_GetHeight());

	paint_SetHeight(16);
	paint_Clear(UNCOLORED);
	paint_DrawStringAt(0, 3, "Humidity:", &Font12, COLORED);
	Epd_SetFrameMemoryPart(paint_GetImage(), 0, 224, paint_GetWidth(), paint_GetHeight());

	paint_SetHeight(24);
	paint_Clear(UNCOLORED);
	sprintf(text, " N/A");
	//sprintf(text, "%i,%i", sensor.humid / 10, sensor.humid % 10);
	paint_DrawStringAt(15, 3, text, &Font24, COLORED);
	Epd_SetFrameMemoryPart(paint_GetImage(), 0, 240, paint_GetWidth(), paint_GetHeight());

	paint_SetHeight(16);
	paint_Clear(UNCOLORED);
	paint_DrawStringAt(0, 3, "Pressure:", &Font12, COLORED);
	Epd_SetFrameMemoryPart(paint_GetImage(), 0, 264, paint_GetWidth(), paint_GetHeight());

	paint_SetHeight(16);
	paint_Clear(UNCOLORED);
	sprintf(text, /*"%i,%i hPa"*/ "N/A hPa");//, (int)(pressure / 100), (int)((pressure / 10) % 10));
	paint_DrawStringAt(0, 3, text, &Font12, COLORED);
	Epd_SetFrameMemoryPart(paint_GetImage(), 24, 280, paint_GetWidth(), paint_GetHeight());


	if (ui.btn_on_screen)
	{
		if (page == GUI_MAIN)
			render_buttons("MAX", "MENU", "MIN");
		else if (page == GUI_MAIN_MAX)
			render_buttons(NULL, "MAIN", "MIN");
		else
			render_buttons("MAX", "MAIN", NULL);
	}
}

// Handle page rendering
void render_screen(void)
{
	if (ui.partial_update)
		Epd_Init(lut_partial_update);
	else
		Epd_Init(lut_full_update);

	switch_clk(E_CLK_FAST_MODE);
	switch (ui.page)
	{
		case GUI_MAIN:
		case GUI_MAIN_MAX:
		case GUI_MAIN_MIN:
			render_main_page(ui.page);
			break;

		case GUI_MENU:
			render_menu_page();
			break;

		case GUI_CLR_STATS:
			render_clr_stat_page();
			break;

		case GUI_GRAPH_HUMID:
		case GUI_GRAPH_PRESS:
		case GUI_GRAPH_TEMP:
			render_graph_page(ui.page);
			break;

		case GUI_TEST_PAGE:
			break;

		default:
			break;
	}
	switch_clk(E_CLK_SLOW_MODE);

	if (message.timer)
		render_message();

	if (!ui.partial_update)
		adc_run();					// Run battery measurement. Should be done after display update?

	Epd_DisplayFrame();			// Show changes to E-ink
	Epd_Sleep();				// Set display to sleep
	ui.partial_update = false;
}

// Handle user interface
void ui_handle(void)
{
	buttons_handle();

	if (ui.screen_update)
	{
		ui.screen_update = false;
		render_screen();				// Update screen
	}
}

// Initialize user interface
void ui_init(void)
{
	Epd_Init(lut_full_update);
	Epd_ClearFrameMemory(0xFF);
	adc_run();
	Epd_DisplayFrame();
	Epd_ClearFrameMemory(0xFF);
	Epd_DisplayFrame();

	init_Paint(image, 0, 0);
	set_page(GUI_MAIN);
}
