#ifndef __FSL_GPIO_H__
#define __FSL_GPIO_H__

#include <fsl_gpio_hal.h>
#include <stdint.h>

/* MACROS */
#if FSL_FEATURE_GPIO_HAS_FAST_GPIO

#define PORT_TYPE			FGPIO_Type *
#define PT_A				FGPIOA
#define PT_B				FGPIOB
#define PT_C				FGPIOC
#define PT_D				FGPIOD
#define PT_E				FGPIOE

#define IN(port, pin)		FGPIO_HAL_ReadPinInput(port, pin)			// get real digital input of pin
#define DIR(port, pin, dir)	FGPIO_HAL_SetPinDir(port, pin, dir)			// set digital pin direction
#define DIR_GET(port, pin)	FGPIO_HAL_GetPinDir(port, pin)				// get digital pin direction
#define SET(port, pin)		FGPIO_HAL_SetPinOutput(port, pin)			// set digital pin value to 1
#define CLR(port, pin)		FGPIO_HAL_ClearPinOutput(port, pin)			// set digital pin value to 0
#define SET_IN(port, pin)	FGPIO_HAL_ReadPinOutput(port, pin) 			// get requested digital state of pin
#define OUT(port, pin, val)	FGPIO_HAL_WritePinOutput(port, pin, val)  	// set digital pin value
#define TOGGLE(port, pin)	FGPIO_HAL_TogglePinOutput(port, pin)  		// set digital pin value its negation

#else

#define PORT_TYPE			GPIO_Type *
#define PT_A				GPIOA
#define PT_B				GPIOB
#define PT_C				GPIOC
#define PT_D				GPIOD
#define PT_E				GPIOE

#define IN(port, pin)		GPIO_HAL_ReadPinInput(port, pin)			// get real digital input of pin
#define DIR(port, pin, dir)	GPIO_HAL_SetPinDir(port, pin, dir)			// set digital pin direction
#define DIR_GET(port, pin)	GPIO_HAL_GetPinDir(port, pin)				// get digital pin direction
#define SET(port, pin)		GPIO_HAL_SetPinOutput(port, pin)			// set digital pin value to 1
#define CLR(port, pin)		GPIO_HAL_ClearPinOutput(port, pin)			// set digital pin value to 0
#define SET_IN(port, pin)	GPIO_HAL_ReadPinOutput(port, pin) 			// get requested digital state of pin
#define OUT(port, pin, val)	GPIO_HAL_WritePinOutput(port, pin, val)  	// set digital pin value
#define TOGGLE(port, pin)	GPIO_HAL_TogglePinOutput(port, pin)  		// set digital pin value its negation

#endif

/* Pindef Generation Macros */
#ifdef __cplusplus
#define __GPIO_FUN() extern "C" inline static
#else
#define __GPIO_FUN() inline static
#endif

#define MAKE_GPIO(name, port, pin) \
	static inline PORT_TYPE name ## _Port() { return port; } \
	static inline uint_fast8_t name ## _Pin() { return pin; } \
	static inline uint_fast8_t name ## _Read() { return IN(port, pin); } \
	static inline uint_fast8_t name ## _ReadSet() { return SET_IN(port, pin); } \
	static inline uint_fast8_t name ## _Write(uint_fast8_t value) { OUT(port, pin,value); return value; } \
	static inline void name ## _Set() { SET(port, pin); } \
	static inline void name ## _Clear() { CLR(port, pin); } \
	static inline void name ## _Toggle() { TOGGLE(port, pin); } \
	static inline gpio_pin_direction_t name ## _DirSet(gpio_pin_direction_t direction) { DIR(port, pin, direction); return direction; } \
	static inline gpio_pin_direction_t name ## _DirGet() { return DIR_GET(port, pin); }

#undef _GLUE
#undef __GLUE
#undef __GPIO_FUN
#endif
