#include "pinmux.h"
#include <fsl_gpio_hal.h>
#include <fsl_port_hal.h>


#define PIN_SHIFT(pin) (1UL << pin)

const PORT_MemMapPtr cPorts[] = {
#ifdef PORTA
		PORTA,
#endif
#ifdef PORTB
		PORTB,
#endif
#ifdef PORTC
		PORTC,
#endif
#ifdef PORTD
		PORTD,
#endif
#ifdef PORTE
		PORTE,
#endif
};

const GPIO_MemMapPtr cGPIOs[] = {
#ifdef GPIOA
		GPIOA,
#endif
#ifdef GPIOB
		GPIOB,
#endif
#ifdef GPIOC
		GPIOC,
#endif
#ifdef GPIOD
		GPIOD,
#endif
#ifdef GPIOE
		GPIOE,
#endif
		};
const IRQn_Type      cIRQs[]  = PORT_IRQS;

void pinmux_init(const pin_struct_t pinDef[], uint32_t pinCount)
{
	uint32_t i;
	GPIO_MemMapPtr GPIO;
	PORT_MemMapPtr PORT;

	SIM_SCGC5 |= ( 0
#ifdef PORTA
			| SIM_SCGC5_PORTA_MASK
#endif
#ifdef PORTB
			| SIM_SCGC5_PORTB_MASK
#endif
#ifdef PORTC
			| SIM_SCGC5_PORTC_MASK
#endif
#ifdef PORTD
			| SIM_SCGC5_PORTD_MASK
#endif
#ifdef PORTE
			| SIM_SCGC5_PORTE_MASK
#endif
			);

	for(i = 0; i < pinCount; i++)
	{
		PORT = cPorts[pinDef[i].port];
		GPIO = cGPIOs[pinDef[i].port];
		PORT_PCR_REG(PORT,pinDef[i].pin) = PORT_PCR_MUX(pinDef[i].mux); /* Pin Mux Control */

		switch (pinDef[i].pe)
		{
		case 1: PORT_PCR_REG(PORT, pinDef[i].pin) |= PORT_PCR_PS_MASK | PORT_PCR_PE_MASK; break;
		case 2: PORT_PCR_REG(PORT, pinDef[i].pin) |= PORT_PCR_PE_MASK; break;
		default: break;
		}

		if(pinDef[i].ds)
				PORT_PCR_REG(PORT,pinDef[i].pin) 	|= PORT_PCR_DSE_MASK;

		if(pinDef[i].mux == ALT1 || pinDef[i].setGPIO)
		{
			GPIO_HAL_WritePinOutput(GPIO, pinDef[i].pin, pinDef[i].value);
			GPIO_HAL_SetPinDir(GPIO, pinDef[i].pin, pinDef[i].mode);
		}

		PORT_PCR_REG(PORT,pinDef[i].pin) |= PORT_PCR_IRQC(pinDef[i].irq); /* Interrupt Configuration */
	}

	for (i = 0; i < (sizeof(cIRQs) / sizeof(IRQn_Type)); i++)
		if (cIRQs[i] != NotAvail_IRQn)
			NVIC_EnableIRQ(cIRQs[i]);
}
