#ifndef __FSL_PINMUX_H__
#define __FSL_PINMUX_H__

#include <fsl_device_registers.h>
#include <stdint.h>

#ifdef __cplusplus
extern "C" {
#endif

/*!Port indices */
typedef enum {
	IO_A = 0,
	IO_B = 1,
	IO_C = 2,
	IO_D = 3,
	IO_E = 4,
} io_port_t;

/*!Pin Initialization Structure */
typedef struct pin_struct
{
	uint8_t		      	port    : 3; /*! Port index */
	uint8_t             pin     : 5; /*! Pin Signal */
	uint16_t		   	irq     : 4; /*! Interrupt Configuration */
	uint16_t           	mux     : 3; /*! Pin Mux Control */
	uint16_t	    	pe      : 2; /*! Pull Enable (0 = off, 1 = up, 2 = down) */
	uint16_t			ds      : 1; /*! Drive Strength Enable */
	uint16_t	    	mode    : 1; /*! Port Data Direction */
	uint16_t			value   : 1; /*! Initial value in case of GPIO */
	uint16_t            setGPIO : 1; /*! GPIO initialization on nonGPIO pin */
} pin_struct_t;

/*! Interrupt Configuration*/
typedef enum
{
    DISABLED = 0, 	/*! Interrupt/DMA request disabled */
    DMA_RIS  = 1,	/*! DMA request on rising edge */
    DMA_FALL = 2,	/*! DMA request on falling edge */
    DMA_BOTH = 3,	/*! DMA request on either edge */
    INT_ZERO = 8,	/*! Interrupt when logic zero */
    INT_RIS  = 9,	/*! Interrupt on rising edge */
    INT_FALL = 10,	/*! Interrupt on falling edge */
    INT_BOTH = 11,	/*! Interrupt on either edge */
    INT_ONE  = 12	/*! Interrupt when logic one */
} interrupt_configuration_t;

/*! Pin Mux Control*/
typedef enum
{
	ALT0 = 0, 	/*! Pin disabled (analog) */
	ALT1,		/*! Alternative 1 (GPIO) */
	ALT2,		/*! Alternative 2 (chip-specific) */
	ALT3,		/*! Alternative 3 (chip-specific) */
	ALT4,		/*! Alternative 4 (chip-specific) */
	ALT5,		/*! Alternative 5 (chip-specific) */
	ALT6,		/*! Alternative 6 (chip-specific) */
	ALT7		/*! Alternative 7 (chip-specific) */
} alt_funcionality_t;

/*!Port indices */
typedef enum {
	NONE   = 0,
	INPUT  = 0,
	OUTPUT = 1,
} gpio_direction_t;

/* Function Prototypes */
void pinmux_init(const pin_struct_t pinDef[], uint32_t pinCount);

#ifdef __cplusplus
}
#endif

#endif
